indexing
	description: "[
			An edge represents a connection between two SIMPLE_NODEs,
			usually	used in a GRAPH.
			]"
	instructions: "[
			There is no "value" feature; a SIMPLE edge contains no "data".
			SIMPLE_EDGEes are used by graphs which contain no data on the 
			edges (see VALUED_GRAPH), but nevertheless need to know about
			connections between nodes.
			
			Note that there is no status feature such as "is_directed".  The
			edge is implied to be directed simply by the definition of `node_from'
			and `node_to'.  This "direction" can be (and is) ignored in classes
			representing undirected graphs see feature `inspect_reachables' from
			SIMPLE_GRAPH for an example of this.)
			]" 
	author: "Jimmy J. Johnson"
	date: "15 Feb 03"

class
	SIMPLE_EDGE
	
inherit
	
	COMPARABLE
--		redefine
--			default_create
--		end

creation
	default_create,
	connect

feature -- Initialization

	connect (a_node_from, a_node_to: like node_from) is
			-- Sets up a connection between the two nodes after 
			-- removing any existing connections.
		require
			node_from_exists: a_node_from /= Void
			node_to_exists: a_node_to /= Void
		do
			default_create
			is_stable := False
			connect_node_from (a_node_from)
			connect_node_to (a_node_to)
			is_stable := True
		ensure
			connections_made: originates_at (a_node_from) and terminates_at (a_node_to)
			symetrical_connections: a_node_from.has (Current) and a_node_to.has (Current)
		end

feature -- Access

	node_from: SIMPLE_NODE
			-- Node at one end of edge and anchor for nodes at each end of the edge

	node_to:  like node_from
			-- Node at other end of edge and anchor for class.
			-- Used "like" 'node_from' so 'node_from' could be used as anchor in
			-- redefinitions.  To give ability to connect differing nodes would need
			-- to define this as NODE and redefine both 'node_from' and 'node_to'.

	other_node (a_node: like node_from): like node_from is
			-- The node at oposite end of the edge from 'a_node'. 
			-- Could be 'a_node' if edge is cyclic.
		require
			good_connection: originates_at (a_node) or else terminates_at (a_node)
		do
			if a_node = node_from then
				Result := node_to
			else
				Result := node_from
			end
		end

feature -- Element change

	connect_node_from (a_node: like node_from) is
			-- Make `node_from' reference `a_node'.
		require
			node_exists: a_node /= Void
		do
			if node_from /= Void then
				if node_from.has (Current) then
					node_from.prune (Current)
				end
			end
			node_from := a_node
			a_node.extend (Current)
		ensure
			connection_made: originates_at (a_node)
			symetrical_connection: a_node.has (Current)
		end

	connect_node_to (a_node: like node_from) is
			-- Make `node_to' reference `a_node'.
		require
			node_exists: a_node /= Void
		do
			if node_to /= Void then
				if node_to.has (Current) then
					node_to.prune (Current)
				end
			end
			node_to := a_node
			a_node.extend (Current)
		ensure
			connection_made: terminates_at (a_node)
			symetrical_connection: a_node.has (Current)
		end

	disconnect is
		-- Routine necessary to keep nodes and edges symetrically connected.
		-- If delete an edge from a node, then the edge is no longer defined
		-- and should also be deleted from the other node.  Setting
		-- 'node_from' and 'node_to' to Void allows the garbage collector
		-- to get rid of edge as long as a reference is not keep to the edge.
		-- There should be no references (other than the calling object) in
		-- this edge or the two involved nodes after this routine is completed. 
		require
			is_connected: is_connected
		do
			is_stable := False
			node_from.prune (Current)
			node_to.prune (Current)
			node_from := Void
			node_to := Void
			is_stable := True
		ensure
			is_disconnected: node_from = Void and node_to = Void	
		end

feature -- Status report
	
	is_connected: BOOLEAN is
			-- True if both ends of edge is connected to a node.
		do
			Result := node_from /= Void and node_to /= Void
		end

	is_disconnected: BOOLEAN is
			-- True is both ends of edge is not connected to a node.
		do
			Result := node_from = Void or node_to = Void
		end

	is_stable: BOOLEAN
			-- Is the edge in a stable state?  That is, does the
			-- edge have a node a both ends?
			-- This is used by invariant.

	is_directed: BOOLEAN
			-- Is the edge directed?  Can the edge only be traversed in
			-- one direction, from the `node_from' to `node_to'?

feature -- Status setting

	set_directed is
			-- Make the edge a "directed" edge by making `is_directed' True.
		do
			is_directed := True
		ensure
			is_directed: is_directed
		end

	set_undirrected is
			-- Make the edge "undirected" by making `is_directed' False.
		do
			is_directed := False
		ensure
			not_is_directed: not is_directed
		end
		
feature -- Querry

	originates_at (a_node: like node_from): BOOLEAN is
			-- Does the edge start at `a_node'?
		do
			Result := node_from = a_node
		end
	
	terminates_at (a_node: like node_from): BOOLEAN is
			-- Does the edge end at `a_node'?
		do
			Result := node_to = a_node
		end
	
	has_connection (a_node: like node_from): BOOLEAN is
			-- Does this edge connect to `a_node'?
			-- Can you traverse the edge from Current to `a_node'?
		do
			Result := terminates_at (a_node) or originates_at (a_node)
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is Current less than other?
		do
			if Current = other then
					-- They are the same node (reference equality)
					-- Defaults to False.
			else
					-- Must check if the edges are connected (i.e. do they
					-- each have nodes at both ends?  See `is_connected'.)
					-- The normal state of an edge is to have a node at each
					-- end, but the order of creation or destruction could
					-- temporarily disrupt this state.  (See class invariant.)
				if is_connected and other.is_connected then
						-- Both edges are connected to two nodes.
						-- Therefore, proceed to check the values in the edges.
					Result := check_nodes (other)
				elseif is_connected and not other.is_connected then
						-- One or both of the ends of the other edge is not
						-- connected to a node, so the other edge goes to
						-- infinity.  Therefore, `Current' must be < `other'.
					Result := True
				elseif not is_connected and other.is_connected then
						-- One or both of the ends of the current edge is not
						-- connected to a node, so the other edge goes to
						-- infinity.  Therefore, `other' is < `Current'.
						-- Defaults to False (i.e. not less than).
				else 
						-- Both are not connected and therefore both are infinite.
						-- Defaults to False (i.e. not less than).
				end
			end
		end

	check_nodes (other: like Current): BOOLEAN is
				-- When checking the nodes (`node_from' and then `node_to') of
				-- Current and `other', is the nodes of Current less than
				-- the nodes of `other'?
				-- Used by `<'.
		require
			other_exists: other /= Void
			current_is_connected: is_connected
			other_is_connected: other.is_connected
		do
			Result := (node_from < other.node_from) or else	
			   	(equal (node_from, other.node_from) and node_to < other.node_to)
		end
		
invariant
	
		-- Either the edge has a node at both ends in which case it is a good edge,
		-- or it is not connected to any node, in preparation to be reclaimed
		-- by garbage collector.
	definition: is_stable implies 
						((node_from /= Void and node_to /= Void) or else
						(node_from = Void and node_to = Void))

end -- class SIMPLE_EDGE
