indexing
	description: "[
			A graph is used to traverse edges from one node to another in a prescribed
			pattern such as pre-order, post-order, breadth-first, etc by starting at
			at root node and continuing to traverse edges to visit nodes until there
			are no more edges.

			After a sort, the graph becomes a list of all the PATHs which were travelled
			based on the sort method.  There are no features for accessing nodes and edges
			which have been traversed; access `path' and then use the features from the path 
			classes.

			In order to have a "directed" graph use `inspect_children' or to "reverse" the
			direction of the edges use `inspect_parents'.  These will make the directivity
			apply to the whole graph.  You can also have a mixture by adding edges for which
			feature `set_directed' has been called.  An example for which this could be 
			used is in a road-map senario, where most streets are two-way, but there is
			an occassional one-way street.
			
			This version uses client relation to store a set of `nodes' and a set
			of `edges'. (Sets are used to prevent duplicate nodes and edges from
			appearing in the graph.)  These features are exported so clients can add
			or remove nodes and edges from the graph at will.  Nodes and edges in these
			sets become "visisble" to the graph sort methods.
			
			Nodes may be connected using the features in the node or edge classes themselves,
			but these connections will not normally be "visible" to the graph.  To make
			edges visible to the graph feature `connect' from this class must be called or
			the edges and nodes must be explicitely added to the `nodes' and `edges' sets.

			Edges and nodes that are not "visible" to the graph will not be traversed or
			visited.  This allows one graph to contain the same nodes as another but the
			traversals can produce different results.
			NOTE:  This can be overriden by feature `inspect_reachables' which makes all
			edges of all the nodes visible.
			
			The proper sort feature is called creating a set of	paths.  These paths are then 
			traversed.  Drawback is the graph must be resorted any time a node or edge is 
			added or removed.
			]"
	instructions: "[
			Be sure to redefine 'new_edge' in order to create edges of the desired type.
			]"
	notes: "[
			Sort methods other than `sort_breadth_first' are not implemented yet.  Also, 
			the recursive breadth-first sort routine is very inefficient and should be redone. 
			
			While sorting the graph by going all the way through it and storing the paths is
			simpler, I now think it would be better if an "internal" cursor were kept and the 
			forth and back features actually moved from node to node.  This could allow the
			sort method to be changed at any time (i.e. step half way through a graph by visiting
			nodes until reaching some condition at which point the user switches to traversing
			each edge.)  This would also make a `is_sorted' feature unnecessary as any `forth'
			operation would be valid no matter what modifications (internally or external) have 
			been made.  The problem would be to move backwards.  What happens if a previously
			visited node is deleted from the graph?  Should the graph, in this case continue back
			until reaching a visited node that is still in the graph?  Is so then going `back' and
			`forth' could have some strange results.
			
			Have `forth', `back', etc. build an internal queue or stack	and sort on the fly, not
			sort the whole thing as done now.  Also, get rid of the inheritance from LINKED_LIST;
			use a reference to a LINKED_LIST [SIMPLE_PATH] instead (I'm exporting only a few features
			anyway from LINKED_LIST anyway.)
			
			If a completely sorted graph is needed, how about a feature `sorted' which can return 
			a sorted copy or `spawn' of Current.
			]" 
	author: "Jimmy J. Johnson"
	date: "2 Mar 03"

deferred class
	SIMPLE_GRAPH
	
inherit
	
		LINKED_LIST [SIMPLE_PATH] -- for access to `item' without cloning (see `path')
			export
				{NONE} all
			redefine
				default_create,
				i_th
			end
		
		LINKED_LIST [SIMPLE_PATH]
			rename
				item as path
			export
				{NONE} 
					all
				{ANY}
					path,
					i_th,
					off,
					valid_index,
					start,
					forth,
					back,
					exhausted
			redefine
				path,
				default_create,
				i_th
			select
				path
			end
		
feature {NONE} -- Initialization

	default_create is
		do
			make
			create nodes.make
			create edges.make
			compare_objects		-- because paths are cloned
			visit_nodes
--			explore_paths
			inspect_relations
--			inspect_reachables
			set_breadth_first
		end

feature -- Access 

	root: SIMPLE_NODE
			-- Node at which to start traversals.
			-- Anchor for node types.
			
	path: SIMPLE_PATH is
			-- Clone of path traversed to get to current node.
			-- Anchor for path types.
			-- (Renamed `item' from LINKED_LIST.)
		do
			Result := clone (Precursor {LINKED_LIST})
				-- cloned so client cannot modify the path
				-- Perhaps no need to clone if PATH is exported correctly,
				-- say as a LINEAR ???
		ensure then
			valid_result: Result /= Void
		end

	new_edge: SIMPLE_EDGE is
			-- Creates a new edge and also makes the Result available
			-- in `last_new_edge'.  There are no nodes with this edge.
			-- Anchor for edge types.
			-- Redefine to create a different type of edge.
		do
			create Result
			last_new_edge := Result
		ensure
			last_new_edge_was_set: last_new_edge = Result
		end
		
--	node: SIMPLE_NODE is
--			-- Node at end of current path.
--		do
--			Result := item.last_node
--		ensure
--			valid_result: Result /= Void
--		end
--		
--	edge: SIMPLE_EDGE is
--			-- Last edge traversed to get to current node.
--			-- Anchor for edge types.
--		require
--			not_alpha_beta_sorted: not is_alphabetical
--			not_off: not off
--			not_at_root: not is_root	-- if at root no edge was traversed.
---- ??? "Not at root" may not be true for some of the sort methods??
--		do
--			Result := item.edge
--		ensure
--			valid_result: Result /= Void
--		end

	nodes: TWO_WAY_SORTED_SET [like root]
			-- Sorted set of all nodes visible to the graph.
			-- Some nodes may not be reachable from 'root'.
			-- Add nodes to this set to make them visible to the graph,

	edges: TWO_WAY_SORTED_SET [like new_edge]
			-- Sorted set of all edges visible to the graph.
			-- Some edges may not be traversable from 'root'.
			-- Add edges to this set to make them visible to the graph,

--	i_th, infix "@" (i: INTEGER): like item is
-- Precursor not allowed for synonym routines
	i_th (i: INTEGER): like path is
			-- Path at `i'-th position
		do
			Result := clone (Precursor {LINKED_LIST} (i))
		ensure then
			valid_result: Result /= Void
		end
	
	unreachable_nodes: LINEAR [like root] is
			-- All the "visible" nodes that can not be reached from root
			-- given the current sort settings.
		do
--			create list.make
-- !!! fix me
		end

	ancestors: LINEAR [like root] is
			-- All ancestors of current node.  That is, all
			-- nodes visited to get to current.
			-- If 'is_inspecting_parents' this will actually
			-- be a list of descendents.
		do
		end

	last_new_edge: like new_edge
			-- Last edge that was created when two nodes were connected.
			-- This is a convenience so a connection can be made followed
			-- by manipulation of the new edge.

--	cursor: CURSOR is
---- ??? do i want this?
--		do
--			create {SIMPLE_GRAPH_CURSOR} Result.make (root, traversal_method, index)
--		end

	index_of_node (a_node: like root; i: INTEGER): INTEGER is
			-- Index of `i'-th occurrence of `a_node' based on last sort.
			-- 0 if none.
		local
			n: SIMPLE_NODE
			pos: INTEGER
			c: INTEGER
		do
			pos := index	-- to save position
			if object_comparison then
				from start
				until exhausted or c = i
				loop
					n := item.last_node
					if a_node.is_equal (n) then
						c := c + 1
					end
					forth
				end
			else
				from start
				until exhausted or c = i
				loop
					n := item.last_node
					if a_node = n then
						c := c + 1
					end
					forth
				end
			end
			if c = i then
				back
				Result := index
			end
			go_i_th (pos)
		end

	index_of_edge (a_edge: like new_edge; i: INTEGER): INTEGER is
			-- Index of 'i'-th occurrence of 'a_edge' based on last sort.
			-- 0 is none.
		local
			e: SIMPLE_EDGE
			pos: INTEGER
			c: INTEGER
		do
			pos := index	-- to save position
			if object_comparison then
				from start
				until exhausted or c = i
				loop
					e := item.edge
					if a_edge.is_equal (e) then
						c := c + 1
					end
					forth
				end
			else
				from start
				until exhausted or c = i
				loop
					e := item.edge
					if a_edge = e then
						c := c + 1
					end
					forth
				end
			end
			if c = i then
				back
				Result := index
			end
			go_i_th (pos)
		end

feature -- Measurement

	level: INTEGER is
			-- Level (or depth) of the current node away from the 'root' node
			-- based on last sort.
			-- The 'root' is at 'level' 0.
			-- A graph with no branches will have at most a level = number of nodes.
		require
			not_off: not off
		do
			Result := item.count
		ensure
			valid_result: Result >= 0 and Result <= count
		end

	depth: INTEGER is
			-- Depth of graph based on last sort.
			-- It is 1 + the highest 'level' on which the GRAPH contains nodes.
			-- Highest level is the node furtherest away (in number of edges 
			-- not necessarily the futherest distance or longest path) from the root.
			-- Should be 1 if graph only has a root node or if sorted alpha-betically.
			-- Can never be more than the number of nodes/paths.
		local
			i: INTEGER
		do
			i := index
			from
				start
			until
				exhausted
			loop
				if level > Result then
					Result := item.count
				end
				forth
			end
			go_i_th (i)
			Result := Result + 1
		ensure
			valid_result: Result >= 1 and Result <= count
		end

	weight: INTEGER is
			-- The number of "leaf" nodes (nodes with no children) in the graph.
			-- This is based on the last sort.
		local
			i: INTEGER
		do
			i := index
			from
				start
			until
				exhausted
			loop
				if is_leaf then
					Result := Result + 1
				end
				forth
			end
			go_i_th (i)
		ensure
			large_enough_result: Result >= 0			-- If the root has no children it is a leaf.
			small_enough_result: Result <= count - 1	-- All the nodes cannot be a leaf node.
		end
		
	breadth: INTEGER is
			-- Number of nodes at current level.
		require
			not_off: not off
		local
			i: INTEGER
			lev: INTEGER
		do
			i := index
			lev := level
			from
				start
			until
				exhausted
			loop
				if item.count = lev then
					Result := Result + 1
				end
				forth
			end
			go_i_th (i)
		ensure
			zero_implies_empty: Result = 0 implies is_empty
			result_large_enough: Result >= 0
			result_small_enough: Result <= count
		end

feature -- Element change

	set_root (a_node: like root) is
			-- Change the `root'
		require
			node_exists: a_node /= Void
			has_node: nodes.has (a_node)
		do
			root := a_node
		ensure
			root_was_set: root = a_node
		end

	extend_node (a_node: like root) is
			-- Make sure 'a_node' is visible to the graph.
			-- The node's edges are not added to the graph.  To
			-- make the edges visible to the graph add them with
			-- 'extend_edge' or use 'deep_extend_node'.
		require
			node_exists: a_node /= Void
		do
			nodes.extend (a_node)
		ensure
			node_inserted: nodes.has (a_node)
		end

	extend_edge (a_edge: like new_edge) is
			-- Make sure 'a_edge' (and its two nodes) is visible to 
			-- the graph.  Only the edge and its nodes will be visible;
			-- not any other connections that may be already present in
			-- the nodes.  To add all connections reachable through either 
			-- of the two nodes of `a_edge', pick one of the two nodes and
			-- use 'deep_extend_node'.
		require
			edge_exists: a_edge /= Void
		do
			nodes.extend (a_edge.node_from)
			nodes.extend (a_edge.node_to)
			edges.extend (a_edge)
		ensure
			first_node_inserted: nodes.has (a_edge.node_from)
			second_node_inserted: nodes.has (a_edge.node_to)
			edge_inserted: edges.has (a_edge)
		end
			
	deep_extend_node (a_node: like root) is
			-- Add 'a_node' and all nodes connected to 'a_node
			-- to the graph.
		local
			n: like root
		do
			nodes.extend (a_node)
			from a_node.start
			until a_node.exhausted
			loop
				n := a_node.edge.other_node (a_node)
				if not nodes.has (n) then 
					deep_extend_node (n)
				end
				a_node.forth
			end
		end

	connect (a_node, a_other_node: like root) is
			-- Make sure graph contains the two nodes and connect them
			-- by creating an edge.  The new edge can be 
			-- accessed through `last_new_edge'.
			-- Does not put descendent nodes into the graph.
		require
			node_exists: a_node /= Void
			other_exists: a_other_node /= Void
		local
			e: like new_edge
		do
			e := new_edge
			e.connect (a_node, a_other_node)
			extend_node (a_node)
			extend_node (a_other_node)
			extend_edge (e)
		ensure
			node_inserted: nodes.has (a_node)
			other_node_inserted: nodes.has (a_other_node)
			edge_inserted: edges.has (last_new_edge)
			proper_from_connection_made: last_new_edge.originates_at (a_node)
			proper_to_connection_made: last_new_edge.terminates_at (a_other_node)
		end

	remove_node is
			-- Remove the current node and its edges from the graph.
			-- The node and its edges remain unchanged they just become
			-- invisible to the graph.
			-- Graph must be sorted for the change to be seen.
		require
			not_off: not off
		do
			prune_node (item.last_node)
			is_sorted := False
		end

	prune_node (a_node: like root) is
			-- Remove 'a_node' and its edges from the graph.
			-- The node and its edges remain unchanged they just become
			-- invisible to the graph.
			-- Graph must be sorted for the change to be seen.
		require
			node_exists: a_node /= Void
		local
			e: SIMPLE_EDGE
		do
			if nodes.has (a_node) then
--				is_sorted := False	-- because fixing to change graph
			end
			from a_node.start
			until a_node.exhausted
			loop
				e := a_node.edge
				prune_edge (e)
				a_node.forth
			end
			nodes.prune (a_node)
		end

	prune_edge (a_edge: like new_edge) is
			-- Remove 'a_edge' from the graph.  The nodes at each
			-- end of the edge will still be visible to the graph.
			-- Graph must be sorted for the change to be seen.
		require
			edge_exists: a_edge /= Void
		do
			if edges.has (a_edge) then
				is_sorted := False	-- because fixing to change graph
			end
			edges.prune (a_edge)
		end

feature -- Status setting

	explore_paths is
			-- Make traversal explore all posible paths.
			-- May traverse edges more than once.
			-- And may visit nodes more than once.
			-- See also `traverse_edges' and `visit_nodes'
		do
			is_exploring_paths := True
			is_traversing_edges := False
			is_visiting_nodes := False
		ensure
			is_expoloring_paths: is_exploring_paths
			not_traversing_edges: not is_traversing_edges
			not_visiting_nodes: not is_visiting_nodes
		end

	traverse_edges is
			-- Make sure all edges will be traversed once.
			-- May visit nodes more than once.
			-- Some paths may go unexplored.
			-- See also `explore_paths' and `visit_nodes'
		do
			is_exploring_paths := False
			is_traversing_edges := True
			is_visiting_nodes := False
		ensure
			not_expoloring_paths: not is_exploring_paths
			is_traversing_edges: is_traversing_edges
			not_visiting_nodes: not is_visiting_nodes
		end
	
	visit_nodes is
			-- Make sure all nodes will be visited once and only once.
			-- Some edges may go untraversed.
			-- Some paths may go unexplored.
			-- See also `traverse_edges' and `explore_paths'
		do
			is_exploring_paths := False
			is_traversing_edges := False
			is_visiting_nodes := True
		ensure
			not_expoloring_paths: not is_exploring_paths
			not_traversing_edges: not is_traversing_edges
			is_visiting_nodes: is_visiting_nodes
		end

	inspect_children is
			-- Make sure traversal will be along edges leaving nodes.
			-- Opposite of `inspect_parents'.
		do
			is_inspecting_children := True
			is_inspecting_parents := False
		ensure
			is_inspecting_children: is_inspecting_children
			not_inspecting_parents: not is_inspecting_parents
		end

	inspect_parents is
			-- Make sure traversal will be along edges entering nodes.
			-- Opposite of `inspect_children'.
			-- Edges for which `is_directed' (from SIMPLE_EDGE) is True will 
			-- still not be traversable.  This is used to override the direction
			-- "implied" by the `node_from' and `node_to' of SIMPLE_EDGE; but not
			-- to override an actually "directed" edge.
		do
			is_inspecting_children := False
			is_inspecting_parents := True
		ensure
			not_inspecting_children: not is_inspecting_children
			not_inspecting_parents: is_inspecting_parents
		end

	inspect_relations is
			-- Make sure traversal will be along any edge no matter its direction.
			-- Both `inspect_children' and `inspect_parents'.
			-- Edges for which `is_directed' (from SIMPLE_EDGE) is True will 
			-- still not be traversable.  This is used to override the direction
			-- "implied" by the `node_from' and `node_to' of SIMPLE_EDGE; but not
			-- to override a actually "directed" edge.
		do
			is_inspecting_children := True
			is_inspecting_parents := True
		ensure
			is_inspecting_children: is_inspecting_children
			is_inspecting_parents: is_inspecting_parents			
		end
		
	inspect_reachables is
			-- Make it possible for traversals to visit node or edges that
			-- are reachable from the root wheather or not they are "visible"
			-- to (or in other words "in") the graph.
			-- This makes it possible to create a graph, set the root node
			-- and do sorts without first putting each node and edge into
			-- the graph itself.
			-- Opposite of `inspect_visibles'.
		do
			is_inspecting_reachables := True
		ensure
			is_inspecting_reachable: is_inspecting_reachables
		end
		
	inspect_visibles is
			-- Make it possible for traversals to visit only thoses nodes
			-- or traverse those edges that are actually "visible" (in other
			-- words "in") the graph.
			-- Some nodes may be reachable from the root but will not show
			-- up in the traversal.
			-- Opposite of `inspect_reachables'.
		do
			is_inspecting_reachables := False
		ensure
			not_inspecting_reachable: not is_inspecting_reachables
		end
			
	set_alphabetical is
			-- Make next `sort' be in alphabetical order
		do
			traversal_method := Alpha
		ensure
			sorting_alphabetically: traversal_method = Alpha
		end
		
	set_breadth_first is
			-- Make next `sort' be in breadth-first order
		do
			traversal_method := Breadth_first
		ensure
			sorting_breadth_first: traversal_method = Breadth_first
		end
		
	set_depth_first is
			-- Make next `sort' be in depth-first order
		do
			traversal_method := Depth_first
		ensure
			sorting_depth_first: traversal_method = Depth_first
		end
		
	set_in_order is
			-- Make next `sort' be in in-order
		do
			traversal_method := In_order
		ensure
			sorting_in_order: traversal_method = In_order
		end
		
	set_post_order is
			-- Make next `sort' be in post-order
		do
			traversal_method := Post_order
		ensure
			sorting_post_order: traversal_method = Post_order
		end
		
	set_pre_order is
			-- Make next `sort' be in pre-order
		do
			traversal_method := Pre_order
		ensure
			sorting_pre_order: traversal_method = Pre_order
		end
		
feature -- Status Report

	is_root: BOOLEAN is
			-- Is current node used as the root during the last sort?
		do
			Result := not off and then item.last_node = root
		end

	is_leaf: BOOLEAN is
			-- Is current node a leaf?
		local
			i: INTEGER
			cn: SIMPLE_NODE		-- current node
		do
			if not off then
					-- True if 'node' appears only at end of paths.
				Result := True
				cn := item.last_node
				i := index
				from start
				until exhausted or not Result
				loop
					if item.has_node (cn) then 
							-- the current node is in path; is it the last?
						if object_comparison then
							Result := item.last_node.is_equal (cn)
						else
							Result := item.last_node = cn
						end
					end
					forth
				end
				go_i_th (i)
			end
		end

	is_exploring_paths: BOOLEAN
			-- Will all posible paths be explored?
			-- May traverse edges more than once.
			-- And may visit nodes more than once.

	is_traversing_edges: BOOLEAN
			-- Will all the edges be traversed once?
			-- May visit nodes more than once.
			-- Some paths may go unexplored.

	is_visiting_nodes: BOOLEAN
			-- Will all nodes be visited once?
			-- Some edges may go untraversed.
			-- Some paths may go unexplored.

	is_inspecting_children: BOOLEAN
			-- Is traversal along the edges leaving nodes?

	is_inspecting_parents: BOOLEAN
			-- Is traverse along edges coming into nodes?

	is_inspecting_relations: BOOLEAN is
			-- Is traversal in both directions regardless of direction of edges?
			-- Is the graph undirected?
		do
			Result := is_inspecting_children and is_inspecting_parents
		end

	is_inspecting_reachables: BOOLEAN
			-- Is a traversal able to visit a node or edge that is "reachable" from
			-- the root wheather it is "visible" (in other words "in") the graph or not.
			-- See `inspect_visibles' and `inspect_reachables'

	is_alphabetical: BOOLEAN is
			-- Was last sort in alphabetical order?
		do
			Result := traversal_method = Alpha
		end

	is_sorted: BOOLEAN
			-- Is the graph in a sorted state?  Adding or removing
			-- nodes or edges will make this false.
-- ??? do I want this?  Can I really tell if it is sorted or not?  Not if
--     edges or nodes are added directly to the sets.

feature -- Duplication

	spawn: like Current is
			-- Create a graph from this one in such a way that
			-- all the nodes and edges visible to this graph will
			-- be visible to Result.  Also, any changes made to the
			-- nodes or edges in this graph or the subgraph will
			-- be evident in the other.  This allows, for example,
			-- a small portion of the graph to be worked with while
			-- making changes visible to the whole.
			-- This is more than a clone but less than a deep_clone.
			-- The new graph is not sorted.
-- ??? does this really work?
		do
			Result := clone (Current)
			root := Void
		end

feature -- Querry

	was_visited (a_node: like root): BOOLEAN is
			-- Is 'a_node' reachable from 'root' after last sort?
		do
			Result := index_of_node (a_node, 1) > 0
		end

	was_traversed (a_edge: like new_edge): BOOLEAN is
			-- Was 'a_edge' traversed on last sort?
		do
			Result := index_of_edge (a_edge, 1) > 0
		end

	was_explored (a_path: like path): BOOLEAN is
			-- Was a path equivelant to 'a_path' explored on last sort
		do
			Result := index_of (a_path, 1) > 0
		end

feature -- Cursor movement

	node_search (a_node: like root) is
			-- Move to first position (at or after current
			-- position) where 'node' and 'a_node' are equal.
			-- If structure does not include 'a_node' ensure that
			-- 'is_exhausted' will be true.
		require
			node_exists: a_node /= Void
		do
			if object_comparison then
				from
				until
					exhausted or else a_node.is_equal (item.last_node)
				loop
					forth
				end
			else
				from
				until
					exhausted or else a_node = item.last_node
				loop
					forth
				end
			end
		ensure
			object_found: (not exhausted and object_comparison) implies equal (a_node, item.last_node);
			item_found: (not exhausted and not object_comparison) implies a_node = item.last_node
		end

feature -- Transformation

	sort is
			-- Sort the graph based on the sort method previously chosen
		do
			inspect traversal_method
			when Alpha then
				sort_alphabetically
			when Breadth_first then
				sort_breadth_first
			when Depth_first then
				sort_depth_first
			when In_order then
				sort_in_order
			when Post_order then
				sort_post_order
			when Pre_order then
				sort_pre_order
			else
			end
		end
		
	sort_alphabetically is
			-- Make traversals go in alpha-betical order.
			-- Call start to restart the traversal.
		local
			p: like item
			temp: TWO_WAY_SORTED_SET [like path]
		do
--			root := Void
			wipe_out
			if is_inspecting_reachables then
				sort_breadth_first
				create temp.make
				from start
				until exhausted
				loop
					create p.make (item.last_node)
					temp.extend (p)
					forth
				end
				wipe_out
				from temp.start
				until temp.exhausted
				loop
					extend (temp.item)
					temp.forth
				end
			else
				from nodes.start
				until nodes.exhausted
				loop
					create p.make (nodes.item)
					extend (p)
					nodes.forth
				end
				start
			end
			traversal_method := Alpha
		end

	sort_in_order is
			-- Visit the nodes by visiting the first adjacent node, then
			-- the node, followed by any remaining adjacent nodes, starting
			-- at `root'.
		require
			root_exits: root /= Void
		do
			traversal_method := In_order
-- Fix me !!
		end

	sort_post_order is
			-- Traverse the graph by visiting leaf nodes before the parent nodes.
			-- How do I account for edges traversed, on the way down or back up?
		require
			root_exits: root /= Void
		do
			traversal_method := Post_order
-- Fix me !!
		end
			
	sort_depth_first is
			-- Traverse the graph by starting at the `root' and traversing the edges in
			-- order all the way to a leaf, visiting the nodes on the way.
		require
			root_exits: root /= Void
		do
			traversal_method := Depth_first
-- Fix me !!
		end

	sort_pre_order is
			-- Set cursor movement so nodes will be visited in pre-order.
			-- This is really the same as `sort_breadth_first'.
		require
			root_exits: root /= Void
		do
			sort_breadth_first
			traversal_method := Pre_order
		end

	sort_breadth_first is
			-- Recursively visit a node then children starting at `root'. 
		require
			root_exits: root /= Void
		local
			p: like path	
		do
			wipe_out
			if is_inspecting_reachables or else nodes.has (root) then
				create p.make (root)
				force (p)
				traverse_path (p)	
			end
			traversal_method := Breadth_first
		end

feature {NONE} -- Implementation

	traverse_path (a_path: like path) is
			-- Recursive procedure used by `sort_breadth_first'.
			-- This needs to be redone.  It is very inefficient.
		require
			path_exists: a_path /= Void
		local
			n, next_n: like root
			e: like new_edge
			p: like item
			c: CURSOR
		do
			n := a_path.last_node
			if not n.sorted then
				n.sort
			end
			from
				n.start
			until
				n.exhausted
			loop
				e := n.edge
				next_n := e.other_node (n)
				-- if not a_path.is_cyclic then
				if not a_path.has (e) then
					p := clone (a_path)
					p.extend (e)
					if should_add_this (p, e, next_n) then
						force (p)
						c := n.cursor
						traverse_path (p)
							-- Restore the cursor position just in case the 
							-- position has been changed during recursive call.						
						n.go_to (c)	
					end
				end
				n.forth
			end
		end

	should_add_this (a_path: like path; a_edge: like new_edge; a_node: like root): BOOLEAN is
			-- Used by traversal routines to determine if `a_path' should be added to the
			-- list of sorted paths.  It checks the "visibility" of `a_node' and `a_edge'
			-- to the graph (i.e. are they in `nodes' and `edges').  It also checks the
			-- statuses of `is_exploring_paths', `is_traversing_edges' and `is_visiting_nodes'
			-- to make sure this path/node/edge has not already been traversed/visited.
			-- Finally it check to make sure the direction of traversal is allowed for this
			-- edge/node combination as on `is_inspecting_parents' and `is_inspecting_children'
			-- and whether or not `a_edge' itself is directed.
		require
			path_exists: a_path /= Void
			edge_exists: a_edge /= Void
			node_exists: a_node /= Void
		do
			Result := 
						-- Is `a_node' and `a_edge' "visible" to the graph?
					(is_inspecting_reachables or else
						(nodes.has (a_node) and then edges.has (a_edge))) and then
						-- Has `a_path' not been explored or `a_edge' not traversed
						-- or `a_node' not visited (depending on the traversal statuses)
					((is_exploring_paths and then not was_explored (a_path)) or
					(is_traversing_edges and then not was_traversed (a_edge)) or 
					(is_visiting_nodes and then not was_visited (a_node))) and then
						-- if so, can the edge be traversed?   
						-- First, check if `a_edge' itself is directed
					(a_edge.is_directed and then a_edge.terminates_at (a_node) or else
						-- then, if not, check if the graph will allow the edge to
						-- be traversed (i.e. is the graph "directed"?)
						(not a_edge.is_directed and then
						((is_inspecting_parents and then a_edge.originates_at(a_node)) or
						(is_inspecting_children and then a_edge.terminates_at(a_node)))))
		end


feature {NONE} -- Implementation

	Alpha, Breadth_first, Depth_first, In_order, Pre_order, Post_order: INTEGER is unique
			-- Possible traversal methods.

	traversal_method: INTEGER
			-- How is the list to be traversed.

invariant
	
	nodes_exists: nodes /= Void
	edges_exists: edges /= Void
	valid_traversal_selected: traversal_method >= Alpha and traversal_method <= Post_order
	
end -- class SIMPLE_GRAPH
