indexing
	description: "[
			Holds values as nodes for use in GRAPHs.
			]"
	instructions: "[
			It can be used as is as a generic container or 'edge' can
			be redefined to change the type of the edges.
			The nodes can have Void values after a 'default_create'.
			]" 
	author: "Jimmy J. Johnson"
	date: "25 Feb 03"

class
	SIMPLE_NODE

inherit
	
	COMPARABLE
		redefine
			default_create
		end

	TWO_WAY_SORTED_SET [SIMPLE_EDGE]
		rename
			make as list_make,
			max as max_edge,
			min as min_edge,
			item as edge,
			count as degree
		undefine
			is_equal
		redefine
			default_create,
			edge,
			wipe_out,
			prune,
			remove
		end
	
create
	default_create

feature {NONE} -- Initialization

	default_create is
		do
			list_make
			allow_adopting
		end

feature -- Access

	edge: SIMPLE_EDGE is
			-- Current edge.
			-- Redefined to make it explicit and add a post-condition.
		do
			Result := Precursor {TWO_WAY_SORTED_SET}
		ensure then
			valid_result: Result /= Void
		end
		
	connections (a_node: like Current): LINEAR [like edge] is
			-- All the edges from/to this node to/from 'a_node'.
		require
			node_exists: a_node /= Void
		local
			list: SORTED_TWO_WAY_LIST [like edge]
		do
			create list.make
			from start
			until exhausted
			loop
				if edge.has_connection (a_node) then
					list.extend (edge)
				end
				forth
			end
			Result := list.linear_representation
		end

	connections_to (a_node: like Current): LINEAR [like edge] is
			-- All the edges from this node to 'a_node'.
		require
			node_exists: a_node /= Void
		local
			list: SORTED_TWO_WAY_LIST [like edge]
		do
			create list.make
			from start
			until exhausted
			loop
				if edge.has_connection (a_node) and then
					edge.terminates_at (a_node) then
					list.extend (edge)
				end
				forth
			end
			Result := list.linear_representation
		end

	parents: TWO_WAY_SORTED_SET [like Current] is
			-- All the (immediate) parent nodes of `Current'
		local
			pos: CURSOR
		do
			create Result.make
			pos := cursor
			from start
			until exhausted
			loop
				if edge.terminates_at (Current) then
					Result.extend (edge.other_node (Current))
				end
				forth
			end
			go_to (pos)
		end
	
	children: TWO_WAY_SORTED_SET [like Current] is
			-- All the immediate descendents of `Current'
		local
			pos: CURSOR
		do
			create Result.make
			pos := cursor
			from start
			until exhausted
			loop
				if edge.originates_at (Current) then
					Result.extend (edge.other_node (Current))
				end
				forth
			end
			go_to (pos)
		end
		
	ancestors: TWO_WAY_SORTED_SET [like Current] is
			-- List of ancestor nodes
		local
--			g: SIMPLE_GRAPH
		do
--			create Result.make
--			create g
--			g.set_pre_order
--			g.inspect_reachables
--			g.inspect_parents
--			g.visit_nodes
--			g.set_root (Current)
--			g.sort
--			from
--				g.start
--			until
--				g.is_exhausted
--			loop
--				Result.extend (g.node)
--				g.forth
--			end
		end

	descendants: TWO_WAY_SORTED_SET [like Current] is
			-- List of ancestor nodes
		local
--			g: SIMPLE_GRAPH
		do
--			create Result.make
--			create g
--			g.set_pre_order
--			g.inspect_reachables
--			g.inspect_children
--			g.visit_nodes
--			g.set_root (Current)
--			g.sort
--			from
--				g.start
--			until
--				g.is_exhausted
--			loop
--				Result.extend (g.node)
--				g.forth
--			end
		end

feature -- Basic operations

	frozen adopt (a_child: like Current) is
			-- Ensure there is a connection (SIMPLE_EDGE) to `a_child'.
			-- Can only adopt same type of nodes.
		require
			can_adopt_children: can_adopt
			child_exists: a_child /= Void
		local
			e: like edge
		do
			if not has_connection_to (a_child) then
				create e.connect (Current, a_child)
			end
		end
		
	frozen deep_adopt (a_child: like Current) is
			-- Adopt `a_child' into Current and into all of the `ancestors'
			-- of Current.
			-- In other words, make `a_child' and all its `descendents'
			-- become descendents of Current and all `ancestors'.
		require
			can_adopt_children: can_adopt
			child_exists: a_child /= Void
		local
			a: TWO_WAY_SORTED_SET [SIMPLE_NODE]
			cd: TWO_WAY_SORTED_SET [SIMPLE_NODE]
		do
			adopt (a_child)
			a := ancestors
			cd := a_child.descendants
			from a.start
			until a.exhausted
			loop
				a.item.adopt (a_child)
				from cd.start
				until cd.exhausted
				loop
					a.item.adopt (cd.item)
					cd.forth
				end
				a.forth
			end
		end

	frozen disown (a_child: like Current) is
			-- Ensure there are no connections (EDGEs) from Current
			-- to `a_child'.
			-- Opposite of `adopt'.
		require
			child_exists: a_child /= Void
		local
			c: LINEAR [SIMPLE_EDGE]
			e: SIMPLE_EDGE
		do
			c := a_child.connections (Current)
			from c.start
			until c.exhausted
			loop
				e := c.item
				if e.terminates_at (a_child) then
					e.disconnect
				end
				c.forth
			end
		end

	frozen deep_disown (a_child: like Current) is
			-- Make sure neither `a_child' or any of its descendants are
			-- descendants of `Current' or of any descendent of `Current'.
			-- Opposite of `deep_adopt'.
		require
			child_exists: a_child /= Void
		local
			d: TWO_WAY_SORTED_SET [SIMPLE_NODE]
			cd: TWO_WAY_SORTED_SET [SIMPLE_NODE]
		do
				-- `Current' disowns `a_child' first
			disown (a_child)
			d := descendants
			cd := a_child.descendants
			from d.start
			until d.exhausted
			loop
					-- Each descendant of `Current' disowns `a_child'
				d.item.disown (a_child)
				from cd.start
				until cd.exhausted
				loop
						-- Each descendant also disowns all descendants of `a_child'
					d.item.disown (cd.item)
					cd.forth
				end
				d.forth
			end
		end

	frozen embrace (a_parent: like Current) is
			-- Make `Current' a child of `a_parent' by
			-- creating an edge.  Equivalent
			-- to `a_parent' adopting `Current'.
		require
			parent_exists: a_parent /= Void
			parent_can_adopt: a_parent.can_adopt
		do
			a_parent.adopt (Current)
		end
		
	frozen deep_embrace (a_parent: like Current) is
			-- Make `Current' and all its descendents children
			-- of `a_parent' and all parent's ancestors.
			-- Equivelant to `a_parent' `deep_adopt'ing Current.
		require
			parent_exists: a_parent /= Void
			parent_can_adopt: a_parent.can_adopt
		do
			a_parent.deep_adopt (Current)
		end
		
	frozen spern (a_parent: like Current) is
			-- Remove any connections from `a_parent' to `Current'
			-- (as in a child rejecting its parents).
			-- Equivelant to `a_parent' disowning `Current'.
		require
			parent_exists: a_parent /= Void
		do
			a_parent.disown (Current)
		end

	frozen deep_spern (a_parent: like Current) is
			-- Remove any connections from `a_parent' to `Current'
			-- and also remove all the descendants of
			-- `Current' from all the `ancestors' of `a_parent' (as if
			-- a child and all its descendants rejected its parents and
			-- all the parent's ancestors).
			-- Equivelant to `a_parent' `deep_disown'ing `Current'.
		require
			parent_exists: a_parent /= Void
		do
			a_parent.deep_disown (Current)
		end

feature -- Measurement

	in_degree: INTEGER is
			-- number of edges ending at this node
		local
			pos: CURSOR
		do
			pos := cursor
			from start
			until exhausted
			loop
				if edge.terminates_at (Current) then
					Result := Result + 1
				end
				forth
			end
			go_to (pos)
		end
	
	out_degree: INTEGER is
			-- number of edges starting at this node
		local
			pos: CURSOR
		do
			pos := cursor
			from start
			until exhausted
			loop
				if edge.originates_at (Current) then
					Result := Result + 1
				end
				forth
			end
			go_to (pos)
		end
	
feature -- Querry

	has_connection (a_node: like Current): BOOLEAN is
			-- Does this node have a connection from or to `a_node'?
		require
			node_exists: a_node /= Void
		local
			pos: CURSOR
		do
			pos := cursor
			from start
			until exhausted or Result
			loop
				if edge.has_connection (a_node) then
					Result := True
				end
				forth
			end
			go_to (cursor)
		end

	has_connection_to (a_node: like Current): BOOLEAN is
			-- Does this node have a conection to 'a_node'.
		require
			node_exists: a_node /= Void
		do
			Result := not connections_to (a_node).is_empty
		end

	is_reachable (a_node: like Current): BOOLEAN is
			-- is a_node reachable from this node
		do
		end

	is_reachable_from (a_node: like Current): BOOLEAN is
			-- Is this node reachable from a_node.
			-- Must explore all paths coming to this node.
		do
		end

feature -- Status report

	can_adopt: BOOLEAN
			-- Can this node become a parent of other node?

feature -- Status setting

	allow_adopting is
			-- Set `can_adopt' to True.
			-- Give ability to adopt children.
		do
			can_adopt := True
		end
			
	forbid_adopting is
			-- Set `can_adopt' to False.
			-- Remove ability to adopt children.
		do
			can_adopt := False
		end
			
feature -- Removal

	remove is
			-- Delete the current edge and therefore delete
			-- the edge from node at other end of edge.
		local
			n: SIMPLE_NODE
			e: SIMPLE_EDGE
		do
			n := edge.other_node (Current)
			e := edge
			precursor {TWO_WAY_SORTED_SET}
			if n.has (e) then
				n.prune (e)
			end
		end

	prune (a_edge: like edge) is
			-- Make sure this node does not contain `a_edge'.
		local
			n: SIMPLE_NODE
			e: SIMPLE_EDGE
		do
			-- If it finds the edge delete the edge from this node 
			-- and also from the connecting node.  Then remove the
			-- nodes from the edge by calling 'disconnect' on the edge.
			search (a_edge)
			if not exhausted then
				n := edge.other_node (Current)
				e := edge
				Precursor (e)
				if n.has (e) then
					n.prune (e)
				e.disconnect
				end
			end
		end
			
	wipe_out is
		do
			from start
			until exhausted
			loop
				remove
			end
			Precursor {TWO_WAY_SORTED_SET}
		end
		
feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
			-- A node with a Void `value' is less than one 
			-- which contains a value.
		do
			if Current = other then
					-- They are the same node (reference equality)
					-- Defaults to False.
			else
				if degree < other.degree then
					Result := True
				elseif other.degree < degree then
					Result := False
				else
						-- Both nodes contain the same number of
						-- edges so check the edges themselves.
					Result := check_edges (other)
				end	
			end
		end

	check_edges (other: like Current): BOOLEAN is
			-- Used by `<' when `degree' equals `other.degree'.
			-- If find an edge in `Current' that is not the same edge
			-- as in `other' (reference equallity) then check those
			-- edges to determine which one is less than the other.
		require
			not_same_node: not (other = Current)
			call_only_if_same_degree: degree = other.degree
		local
			c, other_c: CURSOR
			done: BOOLEAN
		do
			c := cursor
			other_c := other.cursor
			from 
				start
				other.start
			until
				exhausted or Done
			loop
				if not (edge = other.edge) then
					Result := edge < other.edge
					done := True
				end
					-- Continue until find a edge that is different, then
					-- check those two edges to determine result.
					-- No need to check for edges being equal.
					-- The edges can not be the same edge or equal without
					-- originating and terminating at the same nodes.  If 
					-- that were the case than the equality of the nodes
					-- would have already been determined and this feature
					-- would not get called.
				forth
				other.forth
			end
			go_to (cursor)
			other.go_to (other.cursor)
		end
		
feature {NONE} -- Implementation

	has_bogus_edge: BOOLEAN is
			-- Is any edge not connected to this node?  Should not happen.
		local
			i: INTEGER	-- to return to correct spot in list
			e: SIMPLE_EDGE
		do
			i := index
			from start
			until exhausted or Result
			loop
				e := edge
				if (e.is_stable) and then not e.has_connection (Current) then
					Result := True
				end
				forth
			end
			go_i_th (i)			
		end

invariant
	
		-- All items (EDGEs) in current connect to Current.
	symetrical_connections: for_all (agent {SIMPLE_EDGE}.has_connection (Current))
	
end -- class SIMPLE_NODE
