indexing
	description: "[
			An ordered colection of edges beginning at a `start_node'.  It is
			implemented as a LINKED_STACK so edges can only be added and
			removed from the end.
			This is to be used with GRAPHs.
			]"
	instructions: "[
			In order to traverse the list of edges use `linear_representation'.
			To have a path that holds edges and nodes with values redefine
			`edge' and/or `first_node'.
			]" 
	author: "Jimmy J. Johnson"
	date: "26 Feb 03"

class
	SIMPLE_PATH

inherit
	
	COMPARABLE
		redefine
			copy
		end

	LINKED_STACK [SIMPLE_EDGE]
		rename
			item as edge,
			make as list_make,
			fill as join
		export
			{NONE}
				list_make,
				replace
		undefine
			is_equal
		redefine
			edge,
			copy,
			extend,
			force,
			put,
			remove,
			linear_representation,
			join,
			duplicate
		end

create
	make
	
feature {NONE} -- Initialization
	
	make (a_node: like first_node) is
			-- Create an instance
		require
			node_exists: a_node /= Void
		do
			list_make
			first_node := a_node
		end
		
feature -- Access
		
	edge: SIMPLE_EDGE is
			-- The last edge (because path is a stack) and anchor.
			-- To access other edges in path use `linear_representation'.
			-- Redefined to add comment and post-condition.
		do
			Result := Precursor {LINKED_STACK}
		ensure then
			valid_result: Result /= Void
		end
		
	first_node: SIMPLE_NODE
			-- Node at which this path starts and anchor for class
			-- Redefine to change types of nodes the path can hold.

	last_node: like first_node is
			-- The last node in the path.
			-- Since an edge has a node at each end, this feature must start at
			-- the `first_node' and step through the `linear_representation' of
			-- the path choosing the `other_node' (from SIMPLE_EDGE) in order
			-- to descide which of the two nodes of the last edge to chose.
-- FIX ME!!!
	-- a better way would be to start at the last edge, pick a node, and compare it
	-- to the previouse edge.  At some point (sooner) a choice can be made. 
	-- ???  
	-- Think about this.
		local
			lin: LINEAR [SIMPLE_EDGE]
			se: SIMPLE_EDGE
		do
			lin := linear_representation
			Result := first_node
			from
				lin.start
			until
				lin.exhausted
			loop
				se := lin.item
				Result := se.other_node (Result)
				lin.forth
			end
		end

	cost: NUMERIC is
			-- Cost to travel the entire path.
			-- For SIMPLE_PATH, since the edges are SIMPLE_EDGES and do not
			-- have a cost associated with them the result is the number
			-- of edges in current.
		do
			Result := count
		end

	linear_representation: LINEAR_PATH [like edge] is
			-- Representation as a linear structure
		local
			c: CURSOR
		do
				-- (Precusor from LINKED_STACK produces a LIST whose
				-- order is reverse of original order of insertion, so did
				-- not use it.)
--			Result := Precursor {LINKED_STACK}
			c := cursor
			create Result.make (count, first_node)
			from finish
			until exhausted
			loop
				Result.extend (ll_item)
				back
			end
			go_to (c)
			is_changed := False
		end

feature -- Element change

	set_first_node (a_node: like first_node) is
			-- Change the first node.
			-- Also removes any edges from the list, because the edges must
			-- progress in order and there is no garentee that `a_node' 
			-- has a connection to the first one in the list.
		require
			node_exists: a_node /= Void
		do
			first_node := a_node
			wipe_out
		end

	extend (a_edge: like edge) is
			-- Add `a_edge' to the end of the path.  The new edge must
			-- connect to one of the nodes at the end of the edge it will follow.
		do
			check
				valid_edge: a_edge.has_connection (last_node)
			end			
			Precursor {LINKED_STACK} (a_edge)
			is_changed := True
		end

	force, put (a_edge: like edge) is
			-- Add `a_edge' to the end of the path.  The new edge must
			-- connect to one of the nodes at the end of the edge it will follow.
			-- Same as `extend'.
		do
			check
				valid_edge: a_edge.has_connection (last_node)
			end			
			extend (a_edge)
		end

	join (other: like Current) is
			-- Add `other' path to the end of current.
		do
			check 
				joining_allowed: equal (other.first_node, last_node)
			end
			Precursor {LINKED_STACK} (other)
			is_changed := True
		end

	remove is
			-- Remove the item on top and set `is_changed' to true.
		do
			Precursor {LINKED_STACK}
			is_changed := True
		end

feature -- Status report

	is_changed: BOOLEAN
			-- Has the path changed (edge added or removed) since
			-- the last time a `linear_representation' was made?
			-- This limits the number of times a new linear structure
			-- has to be created.
			
feature -- Querry

	has_node (a_node: like first_node): BOOLEAN is
			-- Does the path contain `a_node'?
		require
			node_exists: a_node /= Void
		local
			lin: LINEAR [SIMPLE_EDGE]
			se: SIMPLE_EDGE
		do
			if a_node = first_node then
				Result := True
			elseif not is_empty then
				lin := linear_representation
				from lin.start
				until lin.exhausted or Result
				loop
					se := lin.item
					Result := se.has_connection (a_node)
					forth
				end
			end
		end

	is_cyclic: BOOLEAN is
			-- Does the path contain a cycle?  (i.e. visit a node twice)
--		local
--			c: CURSOR
--			n, test_n: like node
		do
---- FIX ME!!!  this is all messed up.
--			test_n := first_node
--			lin := linear_representation
--			from
--				lin.start
--			until
--				lin.exhausted
--			loop
--				c := lin.cursor
--				from
--				until
--					lin.exhausted
--				loop
--					e := lin.item
--					test_n := e.other_node (test_n)
--					if n = test_n then
--						Result := True
--					end
--					forth
--				end
--				go_to (c)
--				forth
----				n := lin.node
----				test_n := node
--			end
		end




feature -- Duplication

	copy (other: like Current) is
			-- More than a standard copy but less than a deep copy.
			-- Simular to STRING.copy in that it copies the list elements.
			-- but not a deep copy of list elements.
		local
			lin: LINEAR [SIMPLE_EDGE]
		do
			lin := other.linear_representation
			make (other.first_node)
			from
				lin.start
			until
				lin.exhausted
			loop
				extend (lin.item)
				lin.forth
			end
		end

feature -- Duplication

	duplicate (n: INTEGER): like Current is
			-- New path containing the `n' latest items inserted
			-- in current stack.
			-- If `n' is greater than `count', identical to current stack.
		local
			lin: LINEAR [SIMPLE_EDGE]
			i: INTEGER
		do
			create Result.make (first_node)
			lin := linear_representation
			from lin.start
			until lin.exhausted or else i = n
			loop
				Result.extend (lin.item)
				i := i + 1
				lin.forth
			end
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
			-- If the `cost' features of Current and `other' are COMPARABLE
			-- then the the costs are compared; if not then the number of
			-- edges in the two paths are compared.
		local
			c, other_c: COMPARABLE
		do
			c ?= cost
			other_c ?= other.cost
			if c /= Void and other_c /= Void then
					-- The costs can be compared
				Result := c < other_c
			else
				Result := count < other.count
			end
		end

invariant
	
	first_node_exists: first_node /= Void
	
	consistant_edges: linear_representation.for_all (agent {SIMPLE_EDGE}.has_connection (linear_representation.previous_node))
	
end -- class SIMPLE_PATH
