indexing
	description: "[
			Holds values as nodes for use in WEIGHTED_GRAPHs.  The node holds
			WEIGHTED_EDGEs whose cost's are of type E.
			]"
	instructions: "[
			It can be used as is as a generic container or 'edge' can
			be redefined to change the type of the edges.
			The nodes can have Void values after a 'default_create'.
			]" 
	author: "Jimmy J. Johnson"
	date: "15 Feb 03"

class
	WEIGHTED_NODE [E]
	
inherit

	SIMPLE_NODE
		redefine
			edge
		end
	
create
	default_create
	
feature {NONE} -- Initialization

feature -- Access

	edge: WEIGHTED_EDGE [E] is
			-- Anchor for class
			-- Redefined to change type
		do
			Result ?= Precursor {SIMPLE_NODE}
		end
			
feature -- Basic operations

	frozen adopt_weighted (a_child: like Current; a_cost: E) is
			-- Ensure there is a connection (EDGE) with `a_cost' going
			-- from Current to `a_child'.
			-- Can only adopt same type of nodes.
		require
			can_adopt_children: can_adopt
			child_exists: a_child /= Void
			cost_exists: a_cost /= Void
		local
			e: like edge
		do
			create e.make (a_cost)
			edge.connect (Current, a_child)
		end
		
	frozen deep_adopt_weighted (a_child: like Current; a_cost: E) is
			-- Adopt `a_child' into Current and into all of the `ancestors'
			-- of Current using `a_edge_value' as the value on each
			-- new edge connection.
			-- In other words, make `a_child' and all its `descendents'
			-- become descendents of Current and all `ancestors'.
		require
			can_adopt_children: can_adopt
			child_exists: a_child /= Void
			cost_exists: a_cost /= Void
		local
			a: TWO_WAY_SORTED_SET [WEIGHTED_NODE [E]]
			cd: TWO_WAY_SORTED_SET [WEIGHTED_NODE [E]]
		do
			adopt_weighted (a_child, a_cost)
			a := ancestors
			cd := a_child.descendants
			from a.start
			until a.exhausted
			loop
				a.item.adopt_weighted (a_child, a_cost)
				from cd.start
				until cd.exhausted
				loop
					a.item.adopt_weighted (cd.item, a_cost)
					cd.forth
				end
				a.forth
			end
		end

	frozen disown_weighted (a_child: like Current; a_cost: E) is
			-- Ensure there are no connections (EDGEs) from Current
			-- to `a_child' with `a_cost'.
			-- Opposite of `adopt'.
		require
			child_exists: a_child /= Void
			cost_exists: a_cost /= Void
		local
			c: LINEAR [WEIGHTED_EDGE [E]]
			e: WEIGHTED_EDGE [E]
		do
			c := a_child.connections (Current)
			from c.start
			until c.exhausted
			loop
				e := c.item
				if e.terminates_at (a_child) and then equal (e.cost, a_cost) then
					e.disconnect
				end
				c.forth
			end
		end

	frozen deep_disown_weighted (a_child: like Current; a_cost: E) is
			-- Make sure neither `a_child' or any of its descendants are
			-- descendants of `Current' or of any descendent of `Current'
			-- through a connection with value `a_value'.
			-- `a_child' may be a descendent if it is connected with an
			-- edge that does not have value equal to `a_cost'.
			-- Opposite of `deep_adopt'.
		require
			child_exists: a_child /= Void
			cost_exists: a_cost /= Void
		local
			d: TWO_WAY_SORTED_SET [WEIGHTED_NODE [E]]
			cd: TWO_WAY_SORTED_SET [WEIGHTED_NODE [E]]
		do
				-- `Current' disowns `a_child' first
			disown_weighted (a_child, a_cost)
			d := descendants
			cd := a_child.descendants
			from d.start
			until d.exhausted
			loop
					-- Each descendant of `Current' disowns `a_child'
				d.item.disown_weighted (a_child, a_cost)
				from cd.start
				until cd.exhausted
				loop
						-- Each descendant also disowns all descendants of `a_child'
					d.item.disown_weighted (cd.item, a_cost)
					cd.forth
				end
				d.forth
			end
		end

	frozen embrace_weighted (a_parent: like Current; a_cost: E) is
			-- Make `Current' a child of `a_parent' by
			-- creating an edge with `a_cost'.  Equivalent
			-- to `a_parent' adopting `Current'.
		require
			parent_exists: a_parent /= Void
			a_cost_exists: a_cost /= Void
			parent_can_adopt: a_parent.can_adopt
		do
			a_parent.adopt_weighted (Current, a_cost)
		end
		
	frozen deep_embrace_weighted (a_parent: like Current; a_cost: E) is
			-- Make `Current' and all its descendents children
			-- of `a_parent' and all parent's ancestors.
			-- Equivelant to `a_parent' `deep_adopt'ing Current.
		require
			parent_exists: a_parent /= Void
			a_cost_exists: a_cost /= Void
			parent_can_adopt: a_parent.can_adopt
		do
			a_parent.deep_adopt_weighted (Current, a_cost)
		end
		
	frozen spern_weighted (a_parent: like Current; a_cost: E) is
			-- Remove any connections from `a_parent' to `Current' which
			-- have `a_cost' (as in a child rejecting its parents).
			-- Equivelant to `a_parent' disowning `Current'.
		require
			parent_exists: a_parent /= Void
			a_cost_exists: a_cost /= Void
		do
			a_parent.disown_weighted (Current, a_cost)
		end

	frozen deep_spern_weighted (a_parent: like Current; a_cost: E) is
			-- Remove any connections from `a_parent' to `Current' which
			-- have `a_cost' and also remove all the descendants of
			-- `Current' from all the `ancestors' of `a_parent' (as if
			-- a child and all its descendants rejected its parents and
			-- all the parent's ancestors).
			-- Equivelant to `a_parent' `deep_disown'ing `Current'.
		require
			parent_exists: a_parent /= Void
			a_cost_exists: a_cost /= Void
		do
			a_parent.deep_disown_weighted (Current, a_cost)
		end

feature -- Querry

	has_wighted_connection (a_node: like Current; a_cost: E): BOOLEAN is
			-- Does this node have a connection from or to `a_node'
			-- which has a `cost' equal to `a_cost'?
		require
			node_exists: a_node /= Void
			a_cost_exists: a_cost /= Void
		local
			lin: LINEAR [WEIGHTED_EDGE [E]]
		do
			lin := connections (a_node)
			from lin.start
			until Result or else lin.exhausted
			loop
				if equal (lin.item.cost, a_cost) then
					Result := True
				end
				lin.forth
			end
		end

	has_weighted_connection_to (a_node: like Current; a_cost: E): BOOLEAN is
			-- Does this node have a conection to 'a_node' with cost `a_cost'.
		require
			node_exists: a_node /= Void
			a_cost_exists: a_cost /= Void
		local
			lin: LINEAR [WEIGHTED_EDGE [E]]
			e: WEIGHTED_EDGE [E]
		do
			lin := connections_to (a_node)
			from lin.start
			until Result or else lin.exhausted
			loop
				e := lin.item
				if equal (e.cost, a_cost) then
					Result := True
				end
				lin.forth
			end
		end

feature {NONE} -- Implementation

end -- class WEIGHTED_NODE [E]
